@@ -37,18 +37,6 @@
 #include "mach64_drm.h"
 #include "mach64_drv.h"
 
-#define DRM_VTOPHYS(x)   virt_to_bus(x)
-
-#undef DRM_IOREMAP
-#define DRM_IOREMAP(map) \
-(map)->handle=DRM(ioremap) ((map)->offset, (map)->size, (dev) )
-
-#undef DRM_IOREMAPFREE
-#define DRM_IOREMAPFREE(map)                                            \
-        do {                                                            \
-                if ( (map)->handle && (map)->size )                     \
-                        DRM(ioremapfree) ( (map)->handle, (map)->size, (dev) ); \
-        } while (0)
 
 /* ================================================================
  * Engine, FIFO control
@@ -168,7 +156,8 @@ int mach64_do_dma_flush( drm_mach64_private_t *dev_priv )
 	return mach64_ring_idle( dev_priv );
 }
 
-int mach64_do_dma_idle( drm_mach64_private_t *dev_priv ) {
+int mach64_do_dma_idle( drm_mach64_private_t *dev_priv )
+{
 	int ret;
 
 	/* wait for completion */
@@ -458,7 +447,7 @@ static int mach64_bm_dma_test( drm_device_t *dev )
 
 	/* FIXME: get a dma buffer from the freelist here */
 	DRM_DEBUG( "Allocating data memory ...\n" );
-	cpu_addr_data = pci_alloc_consistent( dev->pdev, 0x1000, &data_handle );
+	cpu_addr_data = DRM(pci_alloc)( dev, 0x1000, 0x1000, 0xfffffffful, &data_handle );
 	if (!cpu_addr_data || !data_handle) {
 		DRM_INFO( "data-memory allocation failed!\n" );
 		return DRM_ERR(ENOMEM);
@@ -492,8 +481,7 @@ static int mach64_bm_dma_test( drm_device_t *dev )
 			DRM_INFO( "resetting engine ...\n");
 			mach64_do_engine_reset( dev_priv );
 			DRM_INFO( "freeing data buffer memory.\n" );
-			pci_free_consistent( dev->pdev, 0x1000, 
-					     cpu_addr_data, data_handle );
+			DRM(pci_free)( dev, 0x1000, cpu_addr_data, data_handle );
 			return DRM_ERR(EIO);
 		}
 	}
@@ -535,7 +523,7 @@ static int mach64_bm_dma_test( drm_device_t *dev )
 		DRM_DEBUG( " data[%d] = 0x%08x\n", i, data[i] );
 	}
 
-	DRM_READMEMORYBARRIER();
+	DRM_MEMORYBARRIER();
 
 	DRM_DEBUG( "waiting for idle...\n" );
 	if ( ( i = mach64_do_wait_for_idle( dev_priv ) ) ) {
@@ -547,7 +535,7 @@ static int mach64_bm_dma_test( drm_device_t *dev )
 		MACH64_WRITE( MACH64_PAT_REG0, pat_reg0 );
 		MACH64_WRITE( MACH64_PAT_REG1, pat_reg1 );
 		DRM_INFO( "freeing data buffer memory.\n" );
-		pci_free_consistent( dev->pdev, 0x1000, cpu_addr_data, data_handle );
+		DRM(pci_free)( dev, 0x1000, cpu_addr_data, data_handle );
 		return i;
 	}
 	DRM_DEBUG( "waiting for idle...done\n" );
@@ -584,7 +572,7 @@ static int mach64_bm_dma_test( drm_device_t *dev )
 		MACH64_WRITE( MACH64_PAT_REG0, pat_reg0 );
 		MACH64_WRITE( MACH64_PAT_REG1, pat_reg1 );
 		DRM_INFO( "freeing data buffer memory.\n" );
-		pci_free_consistent( dev->pdev, 0x1000, cpu_addr_data, data_handle );
+		DRM(pci_free)( dev, 0x1000, cpu_addr_data, data_handle );
 		return i;
 	}
 
@@ -612,7 +600,7 @@ static int mach64_bm_dma_test( drm_device_t *dev )
 	MACH64_WRITE( MACH64_PAT_REG1, pat_reg1 );
 
 	DRM_DEBUG( "freeing data buffer memory.\n" );
-	pci_free_consistent( dev->pdev, 0x1000, cpu_addr_data, data_handle );
+	DRM(pci_free)( dev, 0x1000, cpu_addr_data, data_handle );
 	DRM_DEBUG( "returning ...\n" );
 
 	return failed;
@@ -687,6 +675,21 @@ static int mach64_do_dma_init( drm_device_t *dev, drm_mach64_init_t *init )
 		 init->sarea_priv_offset);
 
 	if( !dev_priv->is_pci ) {
+		DRM_FIND_MAP( dev_priv->ring_map, init->ring_offset );
+		if ( !dev_priv->ring_map ) {
+			DRM_ERROR( "can not find ring map!\n" );
+			dev->dev_private = (void *)dev_priv;
+			mach64_do_cleanup_dma(dev);
+			return -EINVAL;
+		}
+		DRM_IOREMAP( dev_priv->ring_map );
+		if ( !dev_priv->ring_map->handle ) {
+		        DRM_ERROR( "can not ioremap virtual address for"
+                                   " descriptor ring\n" );
+			dev->dev_private = (void *) dev_priv;
+			mach64_do_cleanup_dma( dev );
+			return -ENOMEM;
+                }
 	        DRM_FIND_MAP( dev_priv->buffers, init->buffers_offset );
 		if ( !dev_priv->buffers ) {
 			DRM_ERROR( "can not find dma buffer map!\n" );
@@ -705,7 +708,7 @@ static int mach64_do_dma_init( drm_device_t *dev, drm_mach64_init_t *init )
 		DRM_FIND_MAP( dev_priv->agp_textures,
 			      init->agp_textures_offset );
 		if (!dev_priv->agp_textures) {
-			DRM_ERROR("could not find agp texture region!\n");
+			DRM_ERROR( "can not find agp texture region!\n" );
 			dev->dev_private = (void *)dev_priv;
 			mach64_do_cleanup_dma( dev );
 			return DRM_ERR(EINVAL);
@@ -736,31 +739,31 @@ static int mach64_do_dma_init( drm_device_t *dev, drm_mach64_init_t *init )
 		}
 	}
 
-	/* check DMA addressing limitations */
-	DRM_DEBUG( "Setting 32-bit pci dma mask\n" );
-	if ((ret=pci_set_dma_mask( dev->pdev, 0xffffffff ))) {
-		DRM_ERROR( "Setting 32-bit pci dma mask failed\n" );
-		mach64_do_cleanup_dma( dev );
-		return ret;
-	}
-
 	/* allocate descriptor memory from pci pool */
 	DRM_DEBUG( "Allocating dma descriptor ring\n" );
 	dev_priv->ring.size = 0x4000; /* 16KB */
-	dev_priv->ring.start = pci_alloc_consistent( dev->pdev, dev_priv->ring.size, 
-						     &dev_priv->ring.handle );
 
-	if (!dev_priv->ring.start || !dev_priv->ring.handle) {
-		DRM_ERROR( "Allocating dma descriptor ring failed\n");
-		return DRM_ERR(ENOMEM);
-	} else {
-		dev_priv->ring.start_addr = (u32) dev_priv->ring.handle;
-		memset( dev_priv->ring.start, 0x0, 0x4000 );
+	if ( dev_priv->is_pci ) {
+		dev_priv->ring.start = DRM(pci_alloc)( dev, dev_priv->ring.size, 
+						       dev_priv->ring.size, 0xfffffffful,
+						       &dev_priv->ring.handle );
+
+		if (!dev_priv->ring.start || !dev_priv->ring.handle) {
+			DRM_ERROR( "Allocating dma descriptor ring failed\n");
+			return DRM_ERR(ENOMEM);
+		} else {
+			dev_priv->ring.start_addr = (u32) dev_priv->ring.handle;
+		}
+        } else {
+		dev_priv->ring.start = dev_priv->ring_map->handle;
+		dev_priv->ring.start_addr = (u32) dev_priv->ring_map->offset;
 	}
 
+	memset( dev_priv->ring.start, 0, dev_priv->ring.size );
 	DRM_INFO( "descriptor ring: cpu addr 0x%08x, bus addr: 0x%08x\n", 
 		  (u32) dev_priv->ring.start, dev_priv->ring.start_addr );
 
+	ret = 0;
 	if ( dev_priv->driver_mode != MACH64_MODE_MMIO ) {
 
 		/* enable block 1 registers and bus mastering */
@@ -781,11 +784,10 @@ static int mach64_do_dma_init( drm_device_t *dev, drm_mach64_init_t *init )
 		MACH64_WRITE( MACH64_BUS_CNTL, ( MACH64_READ(MACH64_BUS_CNTL) 
 						 | MACH64_BUS_EXT_REG_EN
 						 | MACH64_BUS_MASTER_DIS ) );
-		if ( init->dma_mode == MACH64_MODE_MMIO ) {
+		if ( init->dma_mode == MACH64_MODE_MMIO )
 			DRM_INFO( "Forcing pseudo-DMA mode\n" );
-		} else {
+		else
 			DRM_INFO( "DMA test failed (ret=%d), using pseudo-DMA mode\n", ret );
-		}
 		break;
 	case MACH64_MODE_DMA_SYNC:
 		DRM_INFO( "DMA test succeeded, using synchronous DMA mode\n");
@@ -802,7 +804,6 @@ static int mach64_do_dma_init( drm_device_t *dev, drm_mach64_init_t *init )
 	dev_priv->ring.head = dev_priv->ring.tail = 0;
 	dev_priv->ring.tail_mask = (dev_priv->ring.size / sizeof(u32)) - 1;
 	dev_priv->ring.space = dev_priv->ring.size;
-	dev_priv->ring.high_mark = 128;
 
 	/* setup physical address and size of descriptor table */
 	mach64_do_wait_for_fifo( dev_priv, 1 );
@@ -968,22 +969,30 @@ int mach64_do_cleanup_dma( drm_device_t *dev )
 {
 	DRM_DEBUG( "%s\n", __FUNCTION__ );
 
+#if _HAVE_DMA_IRQ
+	/* Make sure interrupts are disabled here because the uninstall ioctl
+	 * may not have been called from userspace and after dev_private
+	 * is freed, it's too late.
+	 */
+	if ( dev->irq ) DRM(irq_uninstall)(dev);
+#endif
+
 	if ( dev->dev_private ) {
 		drm_mach64_private_t *dev_priv = dev->dev_private;
 
-		/* Discard the allocations for the descriptor table... */
-		if ( (dev->pdev != NULL) && 
-		     (dev_priv->ring.start != NULL) && dev_priv->ring.handle ) {
-			DRM_DEBUG( "freeing dma descriptor ring\n" );
-			pci_free_consistent( dev->pdev, dev_priv->ring.size, 
-					     dev_priv->ring.start, dev_priv->ring.handle );
+		if ( dev_priv->is_pci ) {
+			if ( (dev_priv->ring.start != NULL) && dev_priv->ring.handle ) {
+				DRM(pci_free)( dev, dev_priv->ring.size, 
+					       dev_priv->ring.start, dev_priv->ring.handle );
+			}
+		} else {
+			if ( dev_priv->ring_map )
+				DRM_IOREMAPFREE( dev_priv->ring_map );
 		}
 
-		if ( dev_priv->buffers ) {
+		if ( dev_priv->buffers )
 			DRM_IOREMAPFREE( dev_priv->buffers );
-		}
 
-		DRM_DEBUG( "destroying dma buffer freelist\n" );
 		mach64_destroy_freelist( dev );
 
 		DRM(free)( dev_priv, sizeof(drm_mach64_private_t),
@@ -1005,6 +1014,8 @@ int mach64_dma_init( DRM_IOCTL_ARGS )
 		
 	DRM_DEBUG( "%s\n", __FUNCTION__ );
 
+	LOCK_TEST_WITH_RETURN( dev, filp );
+
 	DRM_COPY_FROM_USER_IOCTL( init, (drm_mach64_init_t *)data, 
 	    sizeof(init) );
 
@@ -1205,7 +1216,8 @@ drm_buf_t *mach64_freelist_get( drm_mach64_private_t *dev_priv )
 						u32 o2 = GETBUFADDR( entry->buf );
 
 						if ( o1 == o2 ) {
-							DRM_ERROR ( "Attempting to free used buffer: i=%d  buf=0x%08x\n", i, o1 );
+							DRM_ERROR ( "Attempting to free used buffer: "
+								    "i=%d  buf=0x%08x\n", i, o1 );
 							mach64_dump_ring_info( dev_priv );
 							return NULL;
 						}
@@ -1216,14 +1228,16 @@ drm_buf_t *mach64_freelist_get( drm_mach64_private_t *dev_priv )
 					list_del(ptr);
 					entry->buf->used = 0;
 					list_add_tail(ptr, &dev_priv->placeholders);
-					DRM_DEBUG( "%s: freed processed buffer (head=%d tail=%d buf ring ofs=%d).\n", __FUNCTION__, head, tail, ofs );
+					DRM_DEBUG( "%s: freed processed buffer (head=%d tail=%d "
+						   "buf ring ofs=%d).\n", __FUNCTION__, head, tail, ofs );
 					return entry->buf;
 				}
 			}
 			DRM_UDELAY( 1 );
 		}
 		mach64_dump_ring_info( dev_priv );
-		DRM_ERROR( "timeout waiting for buffers: ring head_addr: 0x%08x head: %d tail: %d\n", ring->head_addr, ring->head, ring->tail );
+		DRM_ERROR( "timeout waiting for buffers: ring head_addr: 0x%08x head: %d tail: %d\n", 
+			   ring->head_addr, ring->head, ring->tail );
 		return NULL;
 	}
 
